@@ -6,7 +6,7 @@ from fpdf import FPDF
 from datetime import datetime
 import matplotlib.pyplot as plt
 from sklearn.model_selection import StratifiedKFold
-from sklearn.metrics import plot_roc_curve
+from sklearn.metrics import RocCurveDisplay
 from sklearn.metrics import auc
 from urllib.error import URLError
 
@@ -38,6 +38,7 @@ class BinaryDocumenting(FPDF):
         self.cv = None
         self.x = None
         self.y = None
+        self.featureImportances = {}
 
     def header(self):
         try:        # No problem if not possible due lack of internet
@@ -100,7 +101,7 @@ class BinaryDocumenting(FPDF):
         @return:
         @rtype:
         """
-        x, y = self.p.prep_data(self.feature_set)
+        x, y = self.p.x, self.p.y
         self.x, self.y = x.to_numpy(), y.to_numpy()
 
     def analyse(self):
@@ -148,7 +149,7 @@ class BinaryDocumenting(FPDF):
             cm[i] = np.array([[tp, fp], [fn, tn]])
 
             # ROC calculations
-            viz = plot_roc_curve(model, xv, yv, name='ROC fold {}'.format(i + 1), alpha=0.3, ax=ax2)
+            viz = RocCurveDisplay.from_predictions(yv, predictions, name='ROC fold {}'.format(i + 1), ax=ax2)
             interp_tpr = np.interp(mean_fpr, viz.fpr, viz.tpr)
             interp_tpr[0] = 0.0
             true_positive_rates.append(interp_tpr)
@@ -232,17 +233,13 @@ class BinaryDocumenting(FPDF):
         if not os.path.exists(self.p.mainDir + 'EDA/Features/v{}/RF.png'.format(self.p.version)):
             if not os.path.exists(self.p.mainDir + 'EDA/Features/v{}'.format(self.p.version)):
                 os.makedirs(self.p.mainDir + 'EDA/Features/v{}/'.format(self.p.version))
-            from sklearn.ensemble import RandomForestClassifier
-            model = RandomForestClassifier()
-            model.fit(self.p.x, self.p.y)
             fig, ax = plt.subplots(figsize=[4, 6], constrained_layout=True)
             plt.subplots_adjust(left=0.5, top=1, bottom=0)
             ax.spines['right'].set_visible(False)
             ax.spines['bottom'].set_visible(False)
             ax.spines['top'].set_visible(False)
-            ind = np.argsort(model.feature_importances_)
-            plt.barh(list(self.p.x.keys()[ind])[-15:], width=model.feature_importances_[ind][-15:],
-                     color='#2369ec')
+            keys, fi = self.p.featureProcesser.featureImportance['rf']
+            plt.barh(keys[:15], width=fi[:15], color='#2369ec')
             fig.savefig(self.p.mainDir + 'EDA/Features/v{}/RF.png'.format(self.p.version), format='png', dpi=200)
 
     def check_new_page(self, margin=220):
