@@ -10,6 +10,7 @@ import warnings
 import numpy as np
 import pandas as pd
 from tqdm import tqdm
+from pathlib import Path
 from datetime import datetime
 from shap import TreeExplainer
 from shap import KernelExplainer
@@ -132,6 +133,9 @@ class Pipeline:
         self.zScoreThreshold = kwargs.get('z_score_threshold', 4)
         self.includeOutput = kwargs.get('include_output', False)
 
+        # Balancer
+        self.balance = kwargs.get('balance', True)
+
         # Feature Processor
         self.extractFeatures = kwargs.get('extract_features', True)
         self.informationThreshold = kwargs.get('information_threshold', 0.999)
@@ -527,6 +531,51 @@ class Pipeline:
                 print(f"[AutoML] Setting mode to {self.mode} & objective to {self.objective}.")
         return
 
+    @staticmethod
+    def _read_csv(data_path) -> pd.DataFrame:
+        """
+        Read data from given path and set index or multi-index
+
+        Parameters
+        ----------
+        data_path : str or Path
+        """
+        assert Path(data_path).suffix == '.csv', 'Expected a *.csv path'
+
+        data = pd.read_csv(data_path)
+
+        if {'index', 'log'}.issubset(data.columns):
+            # Multi-index: case when IntervalAnalyser was used
+            index = ['log', 'index']
+        elif 'index' in data.columns:
+            index = ['index']
+        else:
+            raise IndexError('No known index was found. '
+                             'Expected to find at least a column named `index`.')
+        return data.set_index(index)
+
+    @staticmethod
+    def _write_csv(data, data_path):
+        """
+        Write data to given path and set index if needed.
+
+        Parameters
+        ----------
+        data : pd.DataFrame or pd.Series
+        data_path : str or Path
+        """
+        assert Path(data_path).suffix == '.csv', 'Expected a *.csv path'
+
+        # Set single-index if not already present
+        if len(data.index.names) == 1 and data.index.name is None:
+            data.index.name = 'index'
+        # Raise error if unnamed index is present
+        if None in data.index.names:
+            raise IndexError(f'Found an unnamed index column ({list(data.index.names)}).')
+
+        # Write data
+        data.to_csv(data_path)
+
     def _data_processing(self, data: pd.DataFrame):
         """
         Organises the data cleaning. Heavy lifting is done in self.dataProcesser, but settings etc. needs
@@ -537,13 +586,16 @@ class Pipeline:
                                            missing_values=self.missingValues,
                                            outlier_removal=self.outlierRemoval, z_score_threshold=self.zScoreThreshold)
 
+        # Set paths
+        data_path = self.mainDir + f'Data/Cleaned_v{self.version}.csv'
+        settings_path = self.mainDir + f'Settings/Cleaning_v{self.version}.json'
+
         try:
             # Load data
-            data = pd.read_csv(self.mainDir + 'Data/Cleaned_v{}.csv'.format(self.version), index_col='index')
+            data = self._read_csv(data_path)
 
             # Load settings
-            self.settings['data_processing'] = json.load(open(self.mainDir + 'Settings/Cleaning_v{}.json'
-                                                              .format(self.version), 'r'))
+            self.settings['data_processing'] = json.load(open(settings_path, 'r'))
             self.dataProcesser.load_settings(self.settings['data_processing'])
 
             if self.verbose > 0:
@@ -554,12 +606,11 @@ class Pipeline:
             data = self.dataProcesser.fit_transform(data)
 
             # Store data
-            data.to_csv(self.mainDir + 'Data/Cleaned_v{}.csv'.format(self.version), index_label='index')
+            self._write_csv(data, data_path)
 
             # Save settings
             self.settings['data_processing'] = self.dataProcesser.get_settings()
-            json.dump(self.settings['data_processing'], open(self.mainDir + 'Settings/Cleaning_v{}.json'
-                                                             .format(self.version), 'w'))
+            json.dump(self.settings['data_processing'], open(settings_path, 'w'))
 
         # If no columns were provided, load them from data processor
         if self.dateCols is None:
@@ -600,14 +651,18 @@ class Pipeline:
         Only run for classification problems. Balances the data using imblearn.
         Does not guarantee to return balanced classes. (Methods are data dependent)
         """
-        # Only necessary for classification
         self.dataSampler = DataSampler(method='both', margin=0.1, cv_splits=self.cvSplits, shuffle=self.shuffle,
                                        fast_run=False, objective=self.objective)
-        if self.mode == 'classification':
+
+        # Set paths
+        data_path = self.mainDir + f'Data/Balanced_v{self.version}.csv'
+
+        # Only necessary for classification
+        if self.mode == 'classification' and self.balance:
             # Check if exists
             try:
                 # Load
-                data = pd.read_csv(self.mainDir + 'Data/Balanced_v{}.csv'.format(self.version), index_col='index')
+                data = self._read_csv(data_path)
 
                 # Split
                 self.y = data[self.target]
@@ -625,7 +680,7 @@ class Pipeline:
                 # Store
                 data = copy.copy(self.x)
                 data[self.target] = self.y
-                data.to_csv(self.mainDir + 'Data/Balanced_v{}.csv'.format(self.version), index_label='index')
+                self._write_csv(data, data_path)
 
     def _sequencing(self):
         """
@@ -634,10 +689,14 @@ class Pipeline:
         """
         self.dataSequencer = Sequencer(back=self.sequenceBack, forward=self.sequenceForward,
                                        shift=self.sequenceShift, diff=self.sequenceDiff)
+
+        # Set paths
+        data_path = self.mainDir + f'Data/Sequence_v{self.version}.csv'
+
         if self.sequence:
             try:
                 # Load data
-                data = pd.read_csv(self.mainDir + 'Data/Sequence_v{}.csv'.format(self.version), index_col='index')
+                data = self._read_csv(data_path)
 
                 # Split and set to memory
                 self.y = data[self.target]
@@ -655,7 +714,7 @@ class Pipeline:
                 # Save
                 data = copy.deepcopy(self.x)
                 data[self.target] = copy.deepcopy(self.y)
-                data.to_csv(self.mainDir + 'Data/Sequence_v{}.csv'.format(self.version), index_label='index')
+                self._write_csv(data, data_path)
 
     def _feature_processing(self):
         """
@@ -665,19 +724,24 @@ class Pipeline:
         self.featureProcesser = FeatureProcesser(mode=self.mode, max_lags=self.maxLags, max_diff=self.maxDiff,
                                                  extract_features=self.extractFeatures, timeout=self.featureTimeout,
                                                  information_threshold=self.informationThreshold)
+
+        # Set paths
+        data_path = self.mainDir + f'Data/Extracted_v{self.version}.csv'
+        settings_path = self.mainDir + f'Settings/Cleaning_v{self.version}.json'
+
         # Check if exists
         try:
             # Loading data
-            self.x = pd.read_csv(self.mainDir + 'Data/Extracted_v{}.csv'.format(self.version), index_col='index')
+            self.x = self._read_csv(data_path)
 
             # Loading settings
-            self.settings['feature_processing'] = json.load(open(self.mainDir + 'Settings/Features_v{}.json'
-                                                                 .format(self.version), 'r'))
+            self.settings['feature_processing'] = json.load(open(settings_path, 'r'))
             self.featureProcesser.load_settings(self.settings['feature_processing'])
             self.featureSets = self.settings['feature_processing']['featureSets']
 
             if self.verbose > 0:
                 print('[AutoML] Loaded Extracted Features')
+
         except FileNotFoundError:
             print('[AutoML] Starting Feature Processor')
 
@@ -685,12 +749,11 @@ class Pipeline:
             self.x, self.featureSets = self.featureProcesser.fit_transform(self.x, self.y)
 
             # Store data
-            self.x.to_csv(self.mainDir + 'Data/Extracted_v{}.csv'.format(self.version), index_label='index')
+            self._write_csv(self.x, data_path)
 
             # Save settings
             self.settings['feature_processing'] = self.featureProcesser.get_settings()
-            json.dump(self.settings['feature_processing'], open(self.mainDir + 'Settings/Features_v{}.json'
-                                                                .format(self.version), 'w'))
+            json.dump(self.settings['feature_processing'], open(settings_path, 'w'))
 
     def _standardizing(self):
         """
@@ -720,11 +783,15 @@ class Pipeline:
         """
         Runs various models to see which work well.
         """
+
+        # Set paths
+        results_path = Path(self.mainDir) / 'Results.csv'
+
         # Load existing results
-        if 'Results.csv' in os.listdir(self.mainDir):
+        if results_path.exists():
 
             # Load results
-            self.results = pd.read_csv(self.mainDir + 'Results.csv')
+            self.results = pd.read_csv(results_path)
 
             # Printing here as we load it
             results = self.results[np.logical_and(
@@ -732,12 +799,12 @@ class Pipeline:
                 self.results['type'] == 'Initial modelling'
             )]
             for fs in set(results['dataset']):
-                print('[AutoML] Initial Modelling for {} ({})'.format(fs, len(self.featureSets[fs])))
+                print(f'[AutoML] Initial Modelling for {fs} ({len(self.featureSets[fs])})')
                 fsr = results[results['dataset'] == fs]
                 for i in range(len(fsr)):
                     row = fsr.iloc[i]
-                    print('[AutoML] {} {}: {:.4f} \u00B1 {:.4f}'.format(row['model'].ljust(40), self.objective,
-                                                                        row['mean_objective'], row['std_objective']))
+                    print(f'[AutoML] {row["model"].ljust(40)} {self.objective}: '
+                          f'{row["mean_objective"]:.4f} \u00B1 {row["std_objective"]:.4f}')
 
         # Check if this version has been modelled
         if self.results is None or self.version not in self.results['version'].values:
@@ -747,9 +814,9 @@ class Pipeline:
 
                 # Skip empty sets
                 if len(cols) == 0:
-                    print('[AutoML] Skipping {} features, empty set'.format(feature_set))
+                    print(f'[AutoML] Skipping {feature_set} features, empty set')
                     continue
-                print('[AutoML] Initial Modelling for {} features ({})'.format(feature_set, len(cols)))
+                print(f'[AutoML] Initial Modelling for {feature_set} features ({len(cols)})')
 
                 # Do the modelling
                 modeller = Modeller(mode=self.mode, shuffle=self.shuffle, store_models=self.storeModels,
@@ -766,7 +833,7 @@ class Pipeline:
                     self.results = self.results.append(results)
 
             # Save results
-            self.results.to_csv(self.mainDir + 'Results.csv', index=False)
+            self.results.to_csv(results_path, index=False)
 
     def grid_search(self, model=None, feature_set: str = None, parameter_set: str = None):
         """
@@ -831,7 +898,7 @@ class Pipeline:
         )])
         for iteration in range(self.gridSearchIterations):
             # Grab settings
-            settings = results.iloc[iteration] # IndexError
+            settings = results.iloc[iteration]  # IndexError
             model = Utils.getModel(settings['model'], mode=self.mode, samples=len(self.x))
             feature_set = settings['dataset']
 
@@ -1018,7 +1085,7 @@ class Pipeline:
         if params is None:
             params = results.iloc[0]['params']
 
-        # Otherwise Find best
+        # Otherwise, find best
         model = results.iloc[0]['model']
         feature_set = results.iloc[0]['dataset']
         params = Utils.parse_json(params)
@@ -1257,9 +1324,9 @@ class Pipeline:
             type(model).__name__, feature_set, len(self.x), len(self.featureSets[feature_set])))
 
         # Cross-Validator
-        cv = StratifiedKFold(n_splits=self.cvSplits, shuffle=self.shuffle)
-        if self.mode == 'regression':
-            cv = KFold(n_splits=self.cvSplits, shuffle=self.shuffle)
+        cv_args = {'n_splits': self.cvSplits, 'shuffle': self.shuffle,
+                   'random_state': 83847939 if self.shuffle else None}
+        cv = KFold(**cv_args) if self.mode == 'regression' else StratifiedKFold(**cv_args)
 
         # Select right hyper parameter optimizer
         if self.gridSearchType == 'base':
@@ -1277,7 +1344,26 @@ class Pipeline:
             raise NotImplementedError('Only Base, Halving and Optuna are implemented.')
         # Get results
         results = grid_search.fit(self.x[self.featureSets[feature_set]], self.y)
-        return results.sort_values('worst_case', ascending=False)
+        results = results.sort_values('worst_case', ascending=False)
+
+        # Warn when best hyperparameters are close to predefined grid
+        edge_params = grid_search.get_parameter_min_max()
+        best_params = pd.Series(results['params'].iloc[0], name='best')
+        params = edge_params.join(best_params, how='inner')
+
+        def warn_when_too_close_to_edge(param: pd.Series, tol=0.01):
+            # Min-max scaling
+            scaled = np.array(param['best']) / (param['max'] - param['min'])
+            # Check if too close and warn if so
+            if not (tol < scaled < 1 - tol):
+                warn_message = ('WARNING: Optimal value for parameter {} '
+                                'is very close to edge case.\t({} = {})'
+                                ).format(param.name, param.name, param['best'])
+                warnings.warn(warn_message)
+
+        params.apply(lambda p: warn_when_too_close_to_edge(p), axis=1)
+
+        return results
 
     def _get_main_predictors(self, data):
         """
