@@ -58,12 +58,6 @@ class TestPipeline(unittest.TestCase):
         pipeline._prepare_production_files(model='StackingRegressor')
         shutil.rmtree('AutoML')
 
-    def test_no_dirs(self):
-        if os.path.exists('AutoML'):
-            shutil.rmtree('AutoML')
-        pipeline = Pipeline(no_dirs=True)
-        assert not os.path.exists('AutoML'), 'Directory created'
-
     def test_regression(self):
         if os.path.exists('AutoML'):
             shutil.rmtree('AutoML')
@@ -111,6 +105,18 @@ class TestPipeline(unittest.TestCase):
         p.load_model(model)
         assert np.allclose(p.predict(self.r_data), prediction)
 
+        # Check settings
+        assert 'pipeline' in settings
+        assert 'version' in settings
+        assert 'name' in settings['pipeline']
+        assert 'model' in settings
+        assert 'amplo_version' in settings
+        assert 'params' in settings
+        assert 'drift_detector' in settings
+        assert 'features' in settings
+        assert 'validation' in settings
+        assert 'data_processing' in settings
+
         # Cleanup
         shutil.rmtree('AutoML')
 
@@ -152,23 +158,24 @@ class TestPipeline(unittest.TestCase):
         p.load_model(model)
         assert np.allclose(p.predict_proba(self.c_data), prediction)
 
+        # Check settings
+        assert 'pipeline' in settings
+        assert 'version' in settings
+        assert 'model' in settings
+        assert 'amplo_version' in settings
+        assert 'params' in settings
+        assert 'drift_detector' in settings
+        assert 'features' in settings
+        assert 'validation' in settings
+        assert 'data_processing' in settings
+
         # Cleanup
         shutil.rmtree('AutoML')
 
-    def test_no_args(self):
-        x, y = make_regression()
-        pipeline = Pipeline(grid_search_iterations=0)
-        pipeline.fit(x, y)
-        assert pipeline.mode == 'regression'
-        shutil.rmtree('AutoML')
-        x, y = make_classification()
-        pipeline = Pipeline(grid_search_iterations=0)
-        pipeline.fit(x, y)
-        assert pipeline.mode == 'classification'
-        pipeline.fit(self.c_data)
-        pipeline.fit(self.r_data)
-        shutil.rmtree('AutoML')
 
     def test_drift(self):
         # todo connect with logger
         pass
+
+    def test_all_models(self):
+        pass
